#!/usr/bin/perl -I installer-items
#
#
#  Copyright (c) 2004, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
#   * Neither the name of SWITCH nor the names of its contributors may be
#     used to endorse or promote products derived from this software without
#     specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.
#
#  $Author: peter $
#
#  $Id: install.pl 45 2006-01-03 15:24:27Z peter $
#
#  $LastChangedRevision: 45 $

require 5.6.0;

use strict;
use CopyRecursive;

my $VERSION = '$Id: install.pl 45 2006-01-03 15:24:27Z peter $';
my $nfsen_version = "1.2.2";

my @ProfileTag = ( 
	"# \n",
	"# Profile live.\n",
	"# \n",
);

my @RequiredDirs = ( 'PREFIX', 'BINDIR', 'LIBEXECDIR', 'CONFDIR', 'DOCDIR', 
					 'VARDIR', 'PROFILESTATDIR', 'PROFILEDATADIR', 
					 'BACKEND_PLUGINDIR', 'FRONTEND_PLUGINDIR', 'HTMLDIR' );

# 
# Get Perl
sub GetPerl {

	my $whichperl;
	my $ans;
	chomp($whichperl = `which perl`);
	if ( length $whichperl ) {
		print "Perl to use: [$whichperl] ";
		chomp($ans = <STDIN>);
		if ( length $ans ) {
			$whichperl = $ans;
		}
	} else {
		print "No Perl found in PATH. Please specify where to find perl [] ";
		chomp($whichperl = <STDIN>);
	}

	while (1) {
		if ( -x $whichperl ) {
			my $err = system("$whichperl -e 'require 5.6.0;'") >> 8;
			last if $err == 0;
			print "Found errors while testing Perl\n";
		} else {
			print "No executable: '$whichperl'\n";
		}
		print "Perl to use: [] ";
		chomp($whichperl = <STDIN>);
	}

	return $whichperl;

} # End of GetPerl

sub GetRRDoffset {

	my $rrd_version = $RRDs::VERSION;
	die "Can't find out which version of RRD you are using!\n" unless defined $rrd_version;

	my $RRDoffset = 0;

	if ( $rrd_version < 1.1 ) { # it's RRD 1.0.x
		$RRDoffset = 77;
	}
	if ( $rrd_version >= 1.2 && $rrd_version < 1.3 ) {
		$RRDoffset = 67;
	}
	if ( !$RRDoffset ) {
		die "RRD version '$rrd_version' not yet supported!\n";
	}
	return $RRDoffset;

} # End of GetRRDoffset

sub CopyDir {
	my $orig = shift;
	my $dest = shift;

	my $DIR;
	mkdir $dest unless -d $dest;
	opendir($DIR, $orig) || die "can't open dir $orig: $!\n";
	while ( my $direntry = readdir $DIR ) {
		next if $direntry eq '.';
		next if $direntry eq '..';
		next if $direntry eq '.svn';
		if ( -d "$orig/$direntry" ) {
			CopyDir("$orig/$direntry", "$dest/$direntry");
			next;
		} 
		if ( -f "$dest/$direntry" ) {
			unlink "$dest/$direntry";
		}
		CopyRecursive::fcopy("$orig/$direntry", "$dest/$direntry");
	}

	closedir $DIR;

} # End of CopyDir

#
# user $user needs to exist and must be a member of $WWWGROUP
sub VerifyUser {
	my $user = shift;

	my $www_gid = getgrnam($NfConf::WWWGROUP);

	my ($login,$pass,$uid,$user_gid) = getpwnam($user);
	if ( !defined $login ) {
		die "user '$user' not found on this system\n";
	}

	# Verify if $user has gid of $WWWGROUP
	if ( $user_gid == $www_gid ) {
		return;
	}

	# Not yet found - check if $user is a member of $WWWGROUP
	my ($gid_name,$passwd,$group_gid,$group_members) = getgrgid($www_gid);
	if ( !defined $gid_name ) {
		die "Group '$user_gid' not found on this system\n";
	}
	# Check the members list
	foreach my $member ( split /\s+/, $group_members ) {
		if ( $member eq $user ) {
			# user found
			return;
		}
	}
	die "User '$user' not a member of group '$NfConf::WWWGROUP'\n";

} # End of VerifyUser

sub VerifyConfig {

	# check if the vars of all required dirs are defined in the Config file
	my $missing = 0;
	foreach my $dir ( @RequiredDirs ) {
		my $var = "NfConf::${dir}";
		no strict 'refs';
		if ( !defined ${$var} ) {
			print stderr "Missing Variable \$$dir in Config File\n";
			$missing++;
		}
		use strict 'refs';
	}
	if ( $missing ) {
		printf stderr "\nSome variables are missing in your config file.\n";
		printf stderr "If you upgrade from an older version of NfSen please add the lines below to your config file.\n";
		printf stderr "See also etc/nfsen-dist.conf for an example.\n\n";
		printf stderr '
# Where to install the binaries
$BINDIR="${BASEDIR}/bin";

#
# Where to install the NfSen Perl modules
$LIBEXECDIR="${BASEDIR}/libexec";

#
# Where to install the config files
$CONFDIR="${BASEDIR}/etc";

#
# Var space for NfSen
$VARDIR="${BASEDIR}/var";

# 
# Where to install the docs
$DOCDIR="${HTMLDIR}/doc";

#
# The Profile stat directory, where all information and RRD DBs of the profile are stored
$PROFILESTATDIR="${BASEDIR}/profiles";

#
# The Profile directory, where all netflow data is stored
$PROFILEDATADIR="${BASEDIR}/profiles";

#
# Where go all the backend plugins
$BACKEND_PLUGINDIR="${BASEDIR}/libexec";

#
# Where go all the frontend plugins
$FRONTEND_PLUGINDIR="${HTMLDIR}/plugins";

';
		exit;
	}

	die "Missing PREFIX directory of nfdump tools!\n" unless defined $NfConf::PREFIX;
	die "'$NfConf::PREFIX' not a directory!\n" unless -d  $NfConf::PREFIX;


	my $wwwgid = getgrnam($NfConf::WWWGROUP) || 
		die "WWW group '$NfConf::WWWGROUP' not found on this system\n";

	VerifyUser($NfConf::WWWUSER);
	VerifyUser($NfConf::USER);

	die "NFSEN does not want to run as root!\n" if $NfConf::USER eq 'root';

	my %_seen;
	foreach my $source ( keys %NfConf::sources ) {
		my $len = length($source);
		if ( $len < 1 || $len > 19 ) {
			die "Error Ident: '$source': Source identifiers must be between 1 and 19 chars long!\n";
		}
		if ( $source =~ /[^a-z0-9_]+/i ) {
			die "Error Ident: '$source': Source identifiers must contain only characters [a-zA-Z0-9_] !\n";
		}
		my $port = $NfConf::sources{$source}{'port'};
		if ( exists $_seen{$port} ) {
			die "Error Ident: '$source': Port $port already configured for source $_seen{$port}!\n";
		} else {
			$_seen{$port} = $source;
		}
	}

} # End of VerifyConfig

sub SetShellParams {

	my $ShellParamFile = "$NfConf::CONFDIR/nfsen-shell-param";

	print "Update Shell vars file $ShellParamFile\n";
	open  SHPARAM, ">$ShellParamFile" || die "Can't open Shell Parameter file: $!\n";
	print SHPARAM "#\n# Automatically generated by $0\n";
	print SHPARAM "PROFILESTATDIR=$NfConf::PROFILESTATDIR\n";
	print SHPARAM "PROFILEDATADIR=$NfConf::PROFILEDATADIR\n";
	print SHPARAM "BINDIR=$NfConf::BINDIR\n";
	print SHPARAM "VARDIR=$NfConf::VARDIR\n";
	print SHPARAM "PREFIX=$NfConf::PREFIX\n";
	print SHPARAM "USER='$NfConf::USER'\n";

	my $tmp = $NfConf::WWWGROUP  ? "'$NfConf::WWWGROUP'"   : "''";
	print SHPARAM "GROUP=$tmp\n";

	$tmp = $NfConf::BUFFLEN ? "'-B $NfConf::BUFFLEN'"   : "''";
	print SHPARAM "BUFFLEN=$tmp\n";

	my @src;
	foreach my $ident ( keys %NfConf::sources ) {
		push @src, "$ident:$NfConf::sources{$ident}{'port'}"
	}
	my $str = join ' ', @src;
	print SHPARAM "SOURCES='$str'\n";
	close SHPARAM;

} # End of SetShellParams

#
# Patching: 
# Any string %%varname%% is replaced by the true value $NfCon::varname from 
# the config file
sub PatchVars {
	my $file = shift;

	open FILE, "$file" || die "Can't open file '$file' for reading: $!\n";
	my @lines = <FILE>;
	close FILE;

	open FILE, ">$file" || die "Can't open file '$file' for writing: $!\n";
	foreach my $line ( @lines ) {
		if ( $line =~ /%%(.+)%%/ ) {
			my $var = "NfConf::$1";
			no strict 'refs';
			$line =~ s#%%(.+)%%#${$var}#;
			use strict 'refs';
		}
		print FILE $line;
	}
	close FILE;

} # End of PatchVars

sub PatchAllScripts {

	my %GlobList = ( 
		"bin/*"		=> 	"$NfConf::BINDIR",
		"libexec/*"	=>	"$NfConf::LIBEXECDIR"
	);

	foreach my $glob_list ( keys %GlobList ) {
		my $dir 	= $GlobList{$glob_list};
    	my @Scripts	= glob($glob_list);
		print "In directory: $dir ...\n";
    	foreach my $script (@Scripts) {
        	$script =~ s#^.+/##;
        	if ( -f "$dir/$script" ) {
            	print "Update script: $script\n";
				PatchVars("$dir/$script");
        	} else {
            	print "ERROR: Failed to update script: '$script'. Installation incomplete!\n";
        	}
    	}
	}

	print "\n";

} # End of PatchAllScripts

sub RenameFiles {

	my @AllProfiles = NfSen::ProfileList();
	if ( scalar @AllProfiles > 0 ) {
		print "Rename Graphic and RRDfiles ... ";
		foreach my $profile ( @AllProfiles ) {
			foreach my $tmp (  '', '_other',  '_udp', '_tcp', '_icmp' ) {
				my $oldfile = "$NfConf::PROFILESTATDIR/$profile/bytes${tmp}.rrd";
				my $newfile = "$NfConf::PROFILESTATDIR/$profile/traffic${tmp}.rrd";
				rename $oldfile, $newfile if -f $oldfile;
			}
			foreach my $tmp (  'day', 'week', 'month',  'year' ) {
				my $oldfile = "$NfConf::PROFILESTATDIR/$profile/bytes-${tmp}.gif";
				my $newfile = "$NfConf::PROFILESTATDIR/$profile/traffic-${tmp}.gif";
				rename $oldfile, $newfile if -f $oldfile;
			}
		}
		print "Done.\n";
	}

} # End of RenameFiles

sub SetupHTML {

	print "Setup php and html files.\n";
	mkdir "$NfConf::HTMLDIR" unless -d "$NfConf::HTMLDIR";
	die "Could not create HTMl directory '$NfConf::HTMLDIR': $!\n"  unless -d $NfConf::HTMLDIR;

	CopyRecursive::dircopy("html", "$NfConf::HTMLDIR");
	open CONF, ">$NfConf::HTMLDIR/conf.php" || die "Can't open conf.php for writing: $!\n";
	print CONF "<?\n";
	print CONF "/* This file was automatically created by the NfSen install.pl script */\n\n";
	print CONF "\$BINDIR     = \"$NfConf::BINDIR\";\n";
	print CONF "\$VARDIR     = \"$NfConf::VARDIR\";\n";
	print CONF "\$DOCURL     = \"/nfsen/doc\";\n";
	print CONF "\$PROFILESTATDIR = \"$NfConf::PROFILESTATDIR\";\n";
	print CONF "\$PROFILEDATADIR = \"$NfConf::PROFILEDATADIR\";\n";
	print CONF "\$FRONTEND_PLUGINDIR = \"$NfConf::FRONTEND_PLUGINDIR\";\n";
	print CONF "\$LIBEXECDIR = \"$NfConf::LIBEXECDIR\";\n";
	print CONF "\$PREFIX     = \"$NfConf::PREFIX\";\n\n";
	print CONF "\$RRDoffset  = \"$NfConf::RRDoffset\";\n\n";
	print CONF "\$SOURCES    = array ( ";
	my @sources =  keys %NfConf::sources;
	for ( my $i=0; $i<scalar @sources; $i++ ) {
		$sources[$i] = "'$sources[$i]'";
	}
	print CONF join ', ', @sources;
	print CONF " );\n";
	print CONF "\n?>\n";
	close CONF;
	print "\n";
	
} # End of SetupHTML

sub SetupEnv {
	
	# Make sure all required directories exist

	umask 0002;
	print "\nSetup diretories:\n";
	my @dirs = (
		"$NfConf::VARDIR",
		"$NfConf::VARDIR/run",
		"$NfConf::VARDIR/filters",
		"$NfConf::PROFILESTATDIR",
		"$NfConf::PROFILESTATDIR/live",
		"$NfConf::PROFILEDATADIR",
		"$NfConf::PROFILEDATADIR/live",
	);

	my ($login,$pass,$uid,$gid) = getpwnam($NfConf::WWWUSER);
	if ( !defined $login ) {
			die "NFSEN user '$NfConf::WWWUSER' not found on this system\n";
	}

	if ( defined $NfConf::WWWGROUP ) {
		$gid  = getgrnam($NfConf::WWWGROUP) || 
			die "NFSEN group '$NfConf::WWWGROUP' not found on this system\n";
	}
	print "\nUse UID/GID $uid $gid\n";

	foreach my $dir ( @dirs ) {
		if ( ! -d $dir ) {
			print "Creating: ";
			mkdir $dir || die "Can't mkdir '$dir': $!\n";
			chmod 0775, $dir || die "Can't chown '$dir': $!\n";
			chown $uid, $gid, $dir || die "Can't chown '$dir': $!\n";
		} else {
			print "Exists: ";
			chmod 0775, $dir || die "Can't chown '$dir': $!\n";
			chown $uid, $gid, $dir || die "Can't chown '$dir': $!\n";
		}
		print "$dir\n";
	}

	print "\nProfile live: spool directories:\n";
	foreach my $ident ( keys %NfConf::sources ) {
		my $dir = "$NfConf::PROFILEDATADIR/live/$ident";
		if ( ! -d $dir ) {
			print "Creating: ";
			mkdir $dir || die "Can't mkdir '$dir': $!\n";
			chmod 0775, $dir || die "Can't chown '$dir': $!\n";
			chown $uid, $gid, $dir || die "Can't chown '$dir': $!\n";
		} else {
			print "Exists: ";
			chmod 0775, $dir || die "Can't chown '$dir': $!\n";
			chown $uid, $gid, $dir || die "Can't chown '$dir': $!\n";
		}
		print "$ident\n";
	}

	RenameFiles();

	my $now = time();
	my $tstart = $now - ( $now % 300 );
	NfSenRRD::SetupRRD("$NfConf::PROFILESTATDIR/live", $tstart - 300, 0);
	if ( $Log::ERROR ) {
		die "Error setup RRD DBs: $Log::ERROR\n";
	}

	my %profileinfo = NfSen::ReadProfile('live');
	if ( $profileinfo{'status'} eq 'empty' ) {
		print "Create profile info for profile 'live'\n";
		# empty - new profile
		$profileinfo{'_comment_'} 	= \@ProfileTag;
		$profileinfo{'name'}		= 'live';
		$profileinfo{'tstart'} 		= $tstart;
		$profileinfo{'tend'} 		= $tstart;
		$profileinfo{'updated'}		= $tstart - 300;
		$profileinfo{'expire'} 		= 0;
		$profileinfo{'maxsize'} 	= 0;
		$profileinfo{'size'} 		= 0;
		$profileinfo{'filter'}		= "";
		$profileinfo{'type'} 		= 0;
		$profileinfo{'locked'} 		= 0;
		$profileinfo{'status'} 		= 'rebuild';
	} else {
		print "Use existing profile info for profile 'live'\n";
	}
	$profileinfo{'sourcelist'}	= join ':', keys %NfConf::sources;

	NfSen::WriteProfile(\%profileinfo);
	my $filelist = "$NfConf::PROFILESTATDIR/live/*rrd $NfConf::PROFILESTATDIR/profile.dat";
	my @AllFIles = glob($filelist);
	chown $uid, $gid, @AllFIles;

	print "\n";

} # End of SetupEnv

sub CopyAllFiles {
	my $ConfigFile = shift;

	print "Copy NfSen dirs etc bin libexec plugins doc ...\n";

	CopyRecursive::dircopy("etc", "$NfConf::CONFDIR");
	CopyRecursive::dircopy("bin", "$NfConf::BINDIR");
	CopyRecursive::dircopy("libexec", "$NfConf::LIBEXECDIR");
	CopyRecursive::dircopy("plugins/backend",  "$NfConf::BACKEND_PLUGINDIR");
	CopyRecursive::dircopy("plugins/frontend", "$NfConf::FRONTEND_PLUGINDIR");
	CopyRecursive::dircopy("doc", "$NfConf::DOCDIR");

	if ( $ConfigFile eq "$NfConf::CONFDIR/nfsen.conf" ) {
		print "Keep config file '$ConfigFile'\n";
	} else {
		print "Copy config file '$ConfigFile'\n";
		CopyRecursive::fcopy("$ConfigFile", "$NfConf::CONFDIR/nfsen.conf");
	}
	print "\n";

} # End of CopyAllFiles

sub Cleanup {
	
	print "Cleanup old files ...\n";

	my @OldFiles = ( "$NfConf::BINDIR/GenGraph.pl" ,
					 "$NfConf::LIBEXECDIR/demoplugin.pm",
					 "$NfConf::LIBEXECDIR/PluginTemplate.pm",
					);

	foreach my $file ( @OldFiles ) {
		unlink $file if -f $file;
	}

} # End of Cleanup

########################
#
# Main starts here
#
########################

my $ConfigFile = shift @ARGV;

# Load the required NfSen modules 
unshift @INC, "libexec";
eval {
	require RRDs; 		import RRDs;
	require NfConf; 	import NfConf;
	require NfSen; 		import NfSen;
	require NfSenRRD; 	import NfSenRRD;
};
if ( $@ ) {
	print "Required nfsen modules not found\n";
	print "$@\n";
	exit;
}

# need to run as root
if ( !NfSen::root_process() ) {
	die "nfsen setup wants to run as root\n";
}

# need a config file
if ( !defined $ConfigFile ) {
	die "Config file required: $0 <nfsen.conf>\n";
}
if ( ! -f $ConfigFile ) {
	die "$ConfigFile: No such file.\n";
}

# Load the config
if ( !NfConf::LoadConfig($ConfigFile) ) {
	print "$Log::ERROR\n";
	exit 1;
}

# 
print "Setup NfSen:\n";
print "Version: $nfsen_version: $VERSION\n\n";

# Get Perl
# Put this into a NfCOnf Variablee, so we can use the standard Patch Procedure
$NfConf::PERL = GetPerl();

$NfConf::RRDoffset = GetRRDoffset();

VerifyConfig();
my $nfsen_run = 0;
if ( -f "$NfConf::VARDIR/run/nfsen-run.pid" ) {
	open PID, "$NfConf::VARDIR/run/nfsen-run.pid" || die "Can't open pid file: $!\n";
	my $pid = <PID>;
	chomp($pid);
	close PID;
	die "Can't extract PID out of '$NfConf::VARDIR/run/nfsen-run.pid'. Stop upgrade" if !defined $pid;
	if ( kill(0, $pid) == 1 ) {
		print "Stop nfsen-run while upgrading .";
		kill 15, $pid;
		my $cnt = 0;
		while ( -f "$NfConf::VARDIR/run/nfsen-run.pid" && $cnt < 300 ) {
			print ".";
			$cnt++;
			sleep(1);
		}
		if ( -f "$NfConf::VARDIR/run/nfsen-run.pid" ) {
			print "\nnfsen-run doesn't want to die! It's not save to upgrade NfSen!\n";
			exit;
		} else {
			print "done.\n";
			$nfsen_run = 1;
		}
	} else {
		print "nfsen-run pid file exists, but no process is running.\n";
		unlink "$NfConf::VARDIR/run/nfsen-run.pid";
	}
}

SetupHTML();
CopyAllFiles($ConfigFile);
SetShellParams();	
PatchAllScripts();
Cleanup();
SetupEnv();

print "Setup done.\n\n";

if ( $nfsen_run ) {
	print "Restart nfsen-run\n";
	system("$NfConf::BINDIR/nfsen-run");
}

print "* You may want to subscribe to the nfsen-discuss mailing list:\n";
print "* http://lists.sourceforge.net/lists/listinfo/nfsen-discuss\n";
print "* Please send bug reports back to me: peter.haag\@switch.ch\n";
exit 0;

